/// <reference types="node" />
import { ec as EC } from 'elliptic';
/**
 * Use types for the `bn.js` lib, e.g. `@types/bn.js`
 */
declare type BNjs = any;
interface CurveOptions {
    curveParameters: number[];
    privatePEMOptions: {
        label: string;
    };
    publicPEMOptions: {
        label: string;
    };
    curve: EC;
}
interface PrivateKeyPKCS1 {
    version: BNjs;
    privateKey: Buffer;
    parameters: number[];
    publicKey?: {
        unused: number;
        data: Buffer;
    };
}
declare type KeyFormat = 'raw' | 'pem' | 'der';
export default class KeyEncoder {
    static ECPrivateKeyASN: any;
    static ECPrivateKey8ASN: any;
    static SubjectPublicKeyInfoASN: any;
    algorithmID: number[];
    options: CurveOptions;
    constructor(options: string | CurveOptions);
    private PKCS1toPKCS8;
    privateKeyObject(rawPrivateKey: string, rawPublicKey: string): PrivateKeyPKCS1;
    publicKeyObject(rawPublicKey: string): {
        algorithm: {
            id: number[];
            curve: number[];
        };
        pub: {
            unused: number;
            data: Buffer;
        };
    };
    encodePrivate(privateKey: string | Buffer, originalFormat: KeyFormat, destinationFormat: KeyFormat, destinationFormatType?: 'pkcs8' | 'pkcs1'): string;
    encodePublic(publicKey: string | Buffer, originalFormat: KeyFormat, destinationFormat: KeyFormat): string;
}
export {};
